@@ -35,8 +35,6 @@
 #include <linux/list.h>
 #include <linux/rwsem.h>
 
-#include <acpi/video.h>
-
 #include "nvstatus.h"
 
 #include "nv-register-module.h"
@@ -61,7 +59,7 @@
 
 #define NVKMS_LOG_PREFIX "nvidia-modeset: "
 
-static bool output_rounding_fix = false;
+static bool output_rounding_fix = true;
 module_param_named(output_rounding_fix, output_rounding_fix, bool, 0400);
 
 /* These parameters are used for fault injection tests.  Normally the defaults
@@ -87,110 +85,6 @@ NvBool nvkms_output_rounding_fix(void)
  * NVKMS interface for nvhost unit for sync point APIs.
  *************************************************************************/
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 #ifdef NVKMS_SYNCPT_STUBS_NEEDED
 /* Unsupported STUB for nvkms_syncpt APIs */
 NvBool nvkms_syncpt_op(
@@ -1062,12 +956,6 @@ nvkms_register_backlight(NvU32 gpu_id, NvU32 display_id, void *drv_priv,
     struct nvkms_backlight_device *nvkms_bd = NULL;
     int i;
 
-#if defined(NV_ACPI_VIDEO_BACKLIGHT_USE_NATIVE)
-    if (!acpi_video_backlight_use_native()) {
-        return NULL;
-    }
-#endif
-
     gpu_info = nvkms_alloc(NV_MAX_GPUS * sizeof(*gpu_info), NV_TRUE);
     if (gpu_info == NULL) {
         return NULL;
@@ -1743,11 +1631,7 @@ module_init(nvkms_init);
 module_exit(nvkms_exit);
 
 #if defined(MODULE_LICENSE)
-
   MODULE_LICENSE("Dual MIT/GPL");
-
-
-
 #endif
 #if defined(MODULE_INFO)
   MODULE_INFO(supported, "external");
