@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2021 NVIDIA Corporation
+    Copyright (c) 2015-2022 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -83,18 +83,7 @@ typedef enum
 
     // ^^^^^^
     // Channel types backed by a CE.
-
-
-
-
-
-
-
-
-
-
     UVM_CHANNEL_TYPE_COUNT = UVM_CHANNEL_TYPE_CE_COUNT,
-
 } uvm_channel_type_t;
 
 typedef enum
@@ -112,34 +101,43 @@ typedef enum
     // There is a single proxy pool and channel per GPU.
     UVM_CHANNEL_POOL_TYPE_CE_PROXY = (1 << 1),
 
-
-
-
-
-
-
-
     UVM_CHANNEL_POOL_TYPE_COUNT = 2,
 
-
     // A mask used to select pools of any type.
     UVM_CHANNEL_POOL_TYPE_MASK  = ((1U << UVM_CHANNEL_POOL_TYPE_COUNT) - 1)
 } uvm_channel_pool_type_t;
 
+typedef enum
+{
+    // Push-based GPFIFO entry
+    UVM_GPFIFO_ENTRY_TYPE_NORMAL,
+
+    // Control GPFIFO entry, i.e., the LENGTH field is zero, not associated with
+    // a push.
+    UVM_GPFIFO_ENTRY_TYPE_CONTROL
+} uvm_gpfifo_entry_type_t;
+
 struct uvm_gpfifo_entry_struct
 {
-    // Offset of the pushbuffer in the pushbuffer allocation used by this entry
+    uvm_gpfifo_entry_type_t type;
+
+    // Channel tracking semaphore value that indicates completion of
+    // this entry.
+    NvU64 tracking_semaphore_value;
+
+    // The following fields are only valid when type is
+    // UVM_GPFIFO_ENTRY_TYPE_NORMAL.
+
+    // Offset of the pushbuffer in the pushbuffer allocation used by
+    // this entry.
     NvU32 pushbuffer_offset;
 
-    // Size of the pushbuffer used for this entry
+    // Size of the pushbuffer used for this entry.
     NvU32 pushbuffer_size;
 
     // List node used by the pushbuffer tracking
     struct list_head pending_list_node;
 
-    // Channel tracking semaphore value that indicates completion of this entry
-    NvU64 tracking_semaphore_value;
-
     // Push info for the pending push that used this GPFIFO entry
     uvm_push_info_t *push_info;
 };
@@ -193,10 +191,10 @@ struct uvm_channel_struct
     // for completion.
     NvU32 gpu_get;
 
-    // Number of currently on-going pushes on this channel
-    // A new push is only allowed to begin on the channel if there is a free
-    // GPFIFO entry for it.
-    NvU32 current_pushes_count;
+    // Number of currently on-going gpfifo entries on this channel
+    // A new push or control GPFIFO is only allowed to begin on the channel if
+    // there is a free GPFIFO entry for it.
+    NvU32 current_gpfifo_count;
 
     // Array of uvm_push_info_t for all pending pushes on the channel
     uvm_push_info_t *push_infos;
@@ -211,30 +209,10 @@ struct uvm_channel_struct
     // been marked as completed.
     struct list_head available_push_infos;
 
-    // GPU tracking semaphore tracking the work in the channel
+    // GPU tracking semaphore tracking the work in the channel.
     // Each push on the channel increments the semaphore, see
     // uvm_channel_end_push().
     uvm_gpu_tracking_semaphore_t tracking_sem;
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 
     // RM channel information
     union
@@ -343,14 +321,6 @@ static bool uvm_channel_is_ce(uvm_channel_t *channel)
     return (channel->pool->pool_type == UVM_CHANNEL_POOL_TYPE_CE) || uvm_channel_is_proxy(channel);
 }
 
-
-
-
-
-
-
-
-
 // Proxy channels are used to push page tree related methods, so their channel
 // type is UVM_CHANNEL_TYPE_MEMOPS.
 static uvm_channel_type_t uvm_channel_proxy_channel_type(void)
@@ -437,8 +407,8 @@ NV_STATUS uvm_channel_reserve_gpu_to_gpu(uvm_channel_manager_t *channel_manager,
                                          uvm_gpu_t *dst_gpu,
                                          uvm_channel_t **channel_out);
 
-// Reserve a specific channel for a push
-NV_STATUS uvm_channel_reserve(uvm_channel_t *channel);
+// Reserve a specific channel for a push or for a control GPFIFO entry.
+NV_STATUS uvm_channel_reserve(uvm_channel_t *channel, NvU32 num_gpfifo_entries);
 
 // Set optimal CE for P2P transfers between manager->gpu and peer
 void uvm_channel_manager_set_p2p_ce(uvm_channel_manager_t *manager, uvm_gpu_t *peer, NvU32 optimal_ce);
@@ -451,6 +421,17 @@ NV_STATUS uvm_channel_begin_push(uvm_channel_t *channel, uvm_push_t *push);
 // Should be used by uvm_push_end() only.
 void uvm_channel_end_push(uvm_push_t *push);
 
+// Write/send a control GPFIFO to channel. This is not supported by proxy
+// channels.
+// Ordering guarantees:
+// Input: Control GPFIFO entries are guaranteed to be processed by ESCHED after
+// all prior GPFIFO entries and pushbuffers have been fetched, but not
+// necessarily completed.
+// Output ordering: A caller can wait for this control entry to complete with
+// uvm_channel_manager_wait(), or by waiting for any later push in the same
+// channel to complete.
+NV_STATUS uvm_channel_write_ctrl_gpfifo(uvm_channel_t *channel, NvU64 ctrl_fifo_entry_value);
+
 const char *uvm_channel_type_to_string(uvm_channel_type_t channel_type);
 const char *uvm_channel_pool_type_to_string(uvm_channel_pool_type_t channel_pool_type);
 
