@@ -806,13 +806,14 @@ NV_STATUS NV_API_CALL nv_dma_map_peer
 (
     nv_dma_device_t *dma_dev,
     nv_dma_device_t *peer_dma_dev,
-    NvU8             bar_index,
+    NvU8             nv_bar_index,
     NvU64            page_count,
     NvU64           *va
 )
 {
     struct pci_dev *peer_pci_dev = to_pci_dev(peer_dma_dev->dev);
     struct resource *res;
+    NvU8 bar_index;
     NV_STATUS status;
 
     if (peer_pci_dev == NULL)
@@ -822,7 +823,7 @@ NV_STATUS NV_API_CALL nv_dma_map_peer
         return NV_ERR_INVALID_REQUEST;
     }
 
-    BUG_ON(bar_index >= NV_GPU_NUM_BARS);
+    bar_index = nv_bar_index_to_os_bar_index(peer_pci_dev, nv_bar_index);
     res = &peer_pci_dev->resource[bar_index];
     if (res->start == 0)
     {
@@ -1089,187 +1090,6 @@ void NV_API_CALL nv_dma_release_sgt
 #endif /* NV_LINUX_DMA_BUF_H_PRESENT && NV_DRM_AVAILABLE && NV_DRM_DRM_GEM_H_PRESENT */
 
 #if defined(NV_LINUX_DMA_BUF_H_PRESENT)
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 #endif /* NV_LINUX_DMA_BUF_H_PRESENT */
 
 #ifndef IMPORT_DMABUF_FUNCTIONS_DEFINED
