@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -315,14 +315,6 @@ namespace DisplayPort
         //
         bool        bDscMstCapBug3143315;
 
-        //
-        // Enable DSC Pass through support in driver based on regkey.
-        //
-        bool        bDscMstEnablePassThrough;
-
-        // Reduce number of 2H1OR LTs which fixes bug 3534707
-        bool        bDscOptimizeLTBug3534707;
-
         //
         // Synaptics branch device doesn't support Virtual Peer Devices so DSC
         // capability of downstream device should be decided based on device's own
@@ -330,12 +322,6 @@ namespace DisplayPort
         //
         bool        bDscCapBasedOnParent;
 
-        //
-        // MST device connnected to dock may issue IRQ for link lost.
-        // Send PowerDown path msg to suppress that.
-        //
-        bool        bPowerDownPhyBeforeD3;
-
         void sharedInit();
         ConnectorImpl(MainLink * main, AuxBus * auxBus, Timer * timer, Connector::EventSink * sink);
 
