@@ -70,6 +70,12 @@ typedef enum _NVHDMIPKT_TYPE
     NVHDMIPKT_TYPE_VENDOR_SPECIFIC_INFOFRAME  = 4,  // VSI
     NVHDMIPKT_TYPE_AUDIO_INFOFRAME            = 5,  // Audio InfoFrame
     NVHDMIPKT_TYPE_EXTENDED_METADATA_PACKET   = 6,  // Extended Metadata Packet (HDMI 2.1)
+    NVHDMIPKT_TYPE_SHARED_GENERIC1            = 7,  // ADA+ Specifc shared generic infoframe 1~6
+    NVHDMIPKT_TYPE_SHARED_GENERIC2            = 8,
+    NVHDMIPKT_TYPE_SHARED_GENERIC3            = 9,
+    NVHDMIPKT_TYPE_SHARED_GENERIC4            = 10,
+    NVHDMIPKT_TYPE_SHARED_GENERIC5            = 11,
+    NVHDMIPKT_TYPE_SHARED_GENERIC6            = 12,
     NVHDMIPKT_INVALID_PKT_TYPE                = 13
 } NVHDMIPKT_TYPE;
 
@@ -99,6 +105,11 @@ typedef enum _NVHDMIPKT_TYPE
 #define NV_HDMI_PKT_TRANSMIT_CTRL_VIDEO_FMT_SW_CTRL     0x00000000
 #define NV_HDMI_PKT_TRANSMIT_CTRL_VIDEO_FMT_HW_CTRL     0x00000001
 
+#define NV_HDMI_PKT_TRANSMIT_CTRL_LOC                   7:6
+#define NV_HDMI_PKT_TRANSMIT_CTRL_LOC_VBLANK            0x00000000
+#define NV_HDMI_PKT_TRANSMIT_CTRL_LOC_VSYNC             0x00000001
+#define NV_HDMI_PKT_TRANSMIT_CTRL_LOC_LOADV             0x00000002
+
 // NVHDMIPKT_TC: HDMI Packet Transmit Control
 // NOTE: Client should use these defines below for transmit control, and avoid using the ones
 //       above. Use only if client knows and wants fine control. And in that case the value
@@ -132,6 +143,29 @@ typedef enum _NVHDMIPKT_TC
     NVHDMIPKT_TRANSMIT_CONTROL_VIDEO_FMT_HW_CTRL           =
     (DRF_DEF(_HDMI_PKT, _TRANSMIT_CTRL, _VIDEO_FMT, _HW_CTRL)),
 
+    NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_EVERY_FRAME_LOC_LOADV=
+    (NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_EVERY_FRAME   |
+     DRF_DEF(_HDMI_PKT, _TRANSMIT_CTRL, _LOC, _LOADV)),
+
+    NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_SINGLE_FRAME_LOC_LOADV=
+    (NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_SINGLE_FRAME  |
+     DRF_DEF(_HDMI_PKT, _TRANSMIT_CTRL, _LOC, _LOADV)),
+
+    NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_EVERY_FRAME_LOC_VSYNC=
+    (NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_EVERY_FRAME   |
+     DRF_DEF(_HDMI_PKT, _TRANSMIT_CTRL, _LOC, _VSYNC)),
+
+    NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_SINGLE_FRAME_LOC_VSYNC=
+    (NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_SINGLE_FRAME  |
+     DRF_DEF(_HDMI_PKT, _TRANSMIT_CTRL, _LOC, _VSYNC)),
+
+    NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_EVERY_FRAME_LOC_VBLANK=
+    (NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_EVERY_FRAME   |
+     DRF_DEF(_HDMI_PKT, _TRANSMIT_CTRL, _LOC, _VBLANK)),
+
+    NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_SINGLE_FRAME_LOC_VBLANK=
+    (NVHDMIPKT_TRANSMIT_CONTROL_ENABLE_SINGLE_FRAME  |
+     DRF_DEF(_HDMI_PKT, _TRANSMIT_CTRL, _LOC, _VBLANK)),
 } NVHDMIPKT_TC;
 
 // RM client handles. Used when client chooses that hdmi library make RM calls.
