@@ -1,25 +1,24 @@
-/*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2020 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- * SPDX-License-Identifier: MIT
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/*******************************************************************************
+    Copyright (c) 2019-2020 NVidia Corporation
+
+    Permission is hereby granted, free of charge, to any person obtaining a copy
+    of this software and associated documentation files (the "Software"), to
+    deal in the Software without restriction, including without limitation the
+    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+    sell copies of the Software, and to permit persons to whom the Software is
+    furnished to do so, subject to the following conditions:
+
+        The above copyright notice and this permission notice shall be
+        included in all copies or substantial portions of the Software.
+
+    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+    DEALINGS IN THE SOFTWARE.
+*******************************************************************************/
 
 #include "nvlink.h"
 #include "nvlink_export.h"
@@ -843,3 +842,147 @@ _nvlink_core_clear_link_state
     link->tx_sublink_state = NVLINK_SUBLINK_STATE_TX_OFF;
     link->rx_sublink_state = NVLINK_SUBLINK_STATE_RX_OFF;
 }
+
+static NvBool
+_nvlink_core_check_if_conn_in_array
+(
+    nvlink_intranode_conn **connArray,
+    NvU32                  connArrayLength,
+    nvlink_intranode_conn *conn
+)
+{
+    NvU32 i;
+
+    for (i = 0; i < connArrayLength; ++i)
+    {
+        if (conn == connArray[i])
+        {
+            return NV_TRUE;
+        }
+    }
+    return NV_FALSE;
+}
+
+NvlStatus
+nvlink_core_powerdown_floorswept_conns_to_off
+(
+    nvlink_link           **links,
+    NvU32                   linkCount,
+    NvU32                   numIoctrls,
+    NvU32                   numLinksPerIoctrl,
+    NvU32                   numActiveLinksPerIoctrl
+)
+{
+    NvU32 i,j;
+    nvlink_intranode_conn **connsToShutdown;
+    nvlink_intranode_conn **visitedConns;
+    nvlink_intranode_conn *conn;
+    NvU32 connCount;
+    NvU32 numConnsToShutdown;
+
+    if (linkCount == 0 || numIoctrls == 0 || numLinksPerIoctrl == 0 ||
+        numActiveLinksPerIoctrl == 0)
+    {
+        return NVL_BAD_ARGS;
+    }
+
+    connsToShutdown = (nvlink_intranode_conn **)nvlink_malloc(
+                            sizeof(nvlink_intranode_conn *) * NVLINK_MAX_SYSTEM_LINK_NUM);
+
+    if (connsToShutdown == NULL)
+    {
+        return NVL_NO_MEM;
+    }
+
+    visitedConns = (nvlink_intranode_conn **)nvlink_malloc(
+                            sizeof(nvlink_intranode_conn *) * NVLINK_MAX_SYSTEM_LINK_NUM);
+
+    if (visitedConns == NULL)
+    {
+        return NVL_NO_MEM;
+    }
+
+
+    //
+    // For each IOCTRL find the total # of connections and shutdown
+    // any connections over the number of active links per IOCTRL
+    //
+    for (i=0; i<numIoctrls; i++)
+    {
+        connCount = 0;
+        numConnsToShutdown = 0;
+        for (j = 0; j < linkCount; j++)
+        {
+            //
+            // If the link is associated with the current IOCTRL
+            // then retrieve its connection. If there is no
+            // active connection associated with the link then continue
+            // If the connection is not active, shutdown the links but don't
+            // increment connCount as this is not an active connection to be used for P2P
+            // If the number of connections found so far is greater
+            // then the number of active links allowed, mark the connection
+            // as being needed to shutdown. Increment the total connection count
+            //
+            if (links[j]->linkNumber >= numLinksPerIoctrl*i &&
+                links[j]->linkNumber < numLinksPerIoctrl*(i+1))
+            {
+                nvlink_core_get_intranode_conn(links[j], &(conn));
+                if (conn == NULL ||
+                    _nvlink_core_check_if_conn_in_array(visitedConns, connCount, conn))
+                {
+                    continue;
+                }
+                else if(nvlink_core_check_intranode_conn_state(conn, NVLINK_LINKSTATE_OFF) ==
+                            NVL_SUCCESS)
+                {
+                    continue;
+                }
+                else if ((nvlink_core_check_intranode_conn_state(conn, NVLINK_LINKSTATE_HS) != NVL_SUCCESS  &&
+                          nvlink_core_check_intranode_conn_state(conn, NVLINK_LINKSTATE_SAFE) != NVL_SUCCESS) &&
+                        (!_nvlink_core_check_if_conn_in_array(connsToShutdown, numConnsToShutdown, conn)))
+                {
+                    // If link is not in SAFE or HS and not currently in our connsToShutdown array then add it
+                    connsToShutdown[numConnsToShutdown++] = conn;
+                }
+                else
+                {
+                    visitedConns[connCount++] = conn;
+                    if (connCount > numActiveLinksPerIoctrl &&
+                        !_nvlink_core_check_if_conn_in_array(connsToShutdown, numConnsToShutdown, conn))
+                    {
+                        connsToShutdown[numConnsToShutdown++] = conn;
+                    }
+                }
+
+                //
+                // If the #of conns found == #of links per IOCTRl then
+                // bail early since we know none of the other links can be
+                // part of this IOCTRL
+                //
+                if (connCount == numLinksPerIoctrl)
+                {
+                    connCount = 0;
+                    break;
+                }
+            }
+        }
+
+        //
+        // If the number of shutdown is non-zero then shutdown the connections
+        // and remove the connection from the corelib since all endpoints
+        // will query the corelib for topology and this connection should no longer
+        // be reported
+        //
+        if (numConnsToShutdown != 0)
+        {
+            nvlink_core_powerdown_intranode_conns_from_active_to_off(connsToShutdown, numConnsToShutdown, 0);
+
+            for (j = 0; j < numConnsToShutdown; ++j)
+            {
+                nvlink_core_remove_intranode_conn(connsToShutdown[j]);
+            }
+        }
+    }
+
+    return NVL_SUCCESS;
+}
