@@ -352,7 +352,7 @@ typedef NVSWITCH_LINK_TYPE  NVSWITCH_LINK_TYPE_LR10;
 //
 
 //
-// LR10 supports CREQ0(0), DNGRD(1), ATR(2), ATSD(3), PROBE(4), RSP0(5), CREQ1(6), and RSP1(7) VCs. 
+// LR10 supports CREQ0(0), DNGRD(1), ATR(2), ATSD(3), PROBE(4), RSP0(5), CREQ1(6), and RSP1(7) VCs.
 // But DNGRD(1), ATR(2), ATSD(3), and PROBE(4) will be never used as PowerPC ATS support is not a POR for LR10 HW.
 //
 #define NVSWITCH_NUM_VCS_LR10    8
@@ -436,7 +436,7 @@ typedef struct
 // In order to collect all the useful unit information into a single container,
 // we need to pick where to find each piece within the parsed discovery table.
 // Top level IP wrappers like NPG have a BCAST range to broadcast reads/writes,
-// but IP sub-units like NPORT have a MULTICAST range within the BCAST IP 
+// but IP sub-units like NPORT have a MULTICAST range within the BCAST IP
 // wrapper to broadcast to all the sub-units in all the IP wrappers.
 // So in the lists below top level IP wrappers (NPG, NVLW, and NXBAR) point
 // to the _BCAST IP wrapper, but sub-unit point to the _MULTICAST range inside
@@ -627,11 +627,11 @@ NvlStatus nvswitch_get_link_public_id_lr10(nvswitch_device *device, NvU32 linkId
 NvlStatus nvswitch_get_link_local_idx_lr10(nvswitch_device *device, NvU32 linkId, NvU32 *localLinkIdx);
 NvlStatus nvswitch_set_training_error_info_lr10(nvswitch_device *device,
                                                 NVSWITCH_SET_TRAINING_ERROR_INFO_PARAMS *pLinkTrainingErrorInfoParams);
+NvlStatus nvswitch_read_vbios_link_entries_lr10(nvswitch_device *device, NvU32 tblPtr,NvU32 entriesCount,NVLINK_CONFIG_DATA_LINKENTRY *link_entries, NvU32 *identified_entriesCount);
 NvlStatus nvswitch_ctrl_get_fatal_error_scope_lr10(nvswitch_device *device, NVSWITCH_GET_FATAL_ERROR_SCOPE_PARAMS *pParams);
 void      nvswitch_init_scratch_lr10(nvswitch_device *device);
 void      nvswitch_init_dlpl_interrupts_lr10(nvlink_link *link);
 NvlStatus nvswitch_init_nport_lr10(nvswitch_device *device);
-NvlStatus nvswitch_get_soe_ucode_binaries_lr10(nvswitch_device *device, const NvU32 **soe_ucode_data, const NvU32 **soe_ucode_header);
 NvlStatus nvswitch_poll_sublink_state_lr10(nvswitch_device *device, nvlink_link *link);
 void      nvswitch_setup_link_loopback_mode_lr10(nvswitch_device *device, NvU32 linkNumber);
 void nvswitch_reset_persistent_link_hw_state_lr10(nvswitch_device *device, NvU32 linkNumber);
@@ -646,10 +646,13 @@ void nvswitch_corelib_get_uphy_load_lr10(nvlink_link *link, NvBool *bUnlocked);
 void      nvswitch_init_buffer_ready_lr10(nvswitch_device *device, nvlink_link *link, NvBool bNportBufferReady);
 NvlStatus nvswitch_ctrl_get_nvlink_lp_counters_lr10(nvswitch_device *device, NVSWITCH_GET_NVLINK_LP_COUNTERS_PARAMS *params);
 NvlStatus nvswitch_service_nvldl_fatal_link_lr10(nvswitch_device *device, NvU32 nvliptInstance, NvU32 link);
+NvlStatus nvswitch_ctrl_inband_send_data_lr10(nvswitch_device *device, NVSWITCH_INBAND_SEND_DATA_PARAMS *p);
+NvlStatus nvswitch_ctrl_inband_read_data_lr10(nvswitch_device *device, NVSWITCH_INBAND_READ_DATA_PARAMS *p);
+NvlStatus nvswitch_launch_ALI_link_training_lr10(nvswitch_device *device, nvlink_link *link);
 NvlStatus nvswitch_service_minion_link_lr10(nvswitch_device *device, NvU32 nvliptInstance);
 void      nvswitch_apply_recal_settings_lr10(nvswitch_device *device, nvlink_link *link);
 NvlStatus nvswitch_ctrl_get_sw_info_lr10(nvswitch_device *device, NVSWITCH_GET_SW_INFO_PARAMS *p);
-void      nvswitch_setup_link_system_registers_lr10(nvswitch_device *device, nvlink_link *link);
-void      nvswitch_load_link_disable_settings_lr10(nvswitch_device *device, nvlink_link *link);
+
+NvlStatus nvswitch_launch_ALI_lr10(nvswitch_device *device);
 
 #endif //_LR10_H_
