@@ -229,15 +229,6 @@
 #define NV_SWITCH_REGKEY_SOE_DISABLE_NO         0x0
 #define NV_SWITCH_REGKEY_SOE_DISABLE_YES        0x1
 
-/*
- * NV_SWITCH_REGKEY_SOE_BOOT_CORE - Selects SOE core
- *
- * Public: Available in release drivers
- */
-#define NV_SWITCH_REGKEY_SOE_BOOT_CORE          "SoeBootCore"
-#define NV_SWITCH_REGKEY_SOE_BOOT_CORE_FALCON   0x0
-#define NV_SWITCH_REGKEY_SOE_BOOT_CORE_DEFAULT  0x2
-
 /*
  * NV_SWITCH_REGKEY_ENABLE_PM
  *
@@ -332,8 +323,6 @@
 #define NV_SWITCH_REGKEY_SPEED_CONTROL_SPEED_40G                0x0F
 #define NV_SWITCH_REGKEY_SPEED_CONTROL_SPEED_50G                0x10
 #define NV_SWITCH_REGKEY_SPEED_CONTROL_SPEED_53_12500G          0x11
-#define NV_SWITCH_REGKEY_SPEED_CONTROL_SPEED_100_00000G         0x12
-#define NV_SWITCH_REGKEY_SPEED_CONTROL_SPEED_106_25000G         0x13
 
 /*
  * Enable/Disable periodic flush to inforom. Default is disabled.
@@ -411,11 +400,18 @@
  *
  * When the regkey is set to FALCON, the Nvswitch driver will run MINION on Falcon core.
  *
+ * If set to RISCV, the MINION will run on RISCV core in Non-Manifest Mode.
+ * If set to RISCV_MANIFEST, the MINION will run on RISCV core in Manifest Mode.
+ *
+ * In the default option, RISCV_BCR_CTRL register will be used to get the default core.
+ *
  * Private: Debug use only
  */
 #define NV_SWITCH_REGKEY_MINION_SET_UCODE_TARGET                "MinionSetUcodeTarget"
 #define NV_SWITCH_REGKEY_MINION_SET_UCODE_TARGET_DEFAULT        0x0
 #define NV_SWITCH_REGKEY_MINION_SET_UCODE_TARGET_FALCON         0x1
+#define NV_SWITCH_REGKEY_MINION_SET_UCODE_TARGET_RISCV          0x2
+#define NV_SWITCH_REGKEY_MINION_SET_UCODE_TARGET_RISCV_MANIFEST 0x3
 
 /*
  * NV_SWITCH_REGKEY_MINION_SET_SIMMODE - Selects simmode settings to send to MINION
@@ -502,13 +498,14 @@
 /*
  * NV_SWITCH_REGKEY_LINK_TRAINING_SELECT - Select the Link training to be done
  *
- * This regkey will
+ * For LS10, links can be trained via non-ALI or ALI training. This regkey will
  * allow for overriding System Defaults and can force either training method
  * when desired.
  */
-
 #define NV_SWITCH_REGKEY_LINK_TRAINING_SELECT           "LinkTrainingMode"
 #define NV_SWITCH_REGKEY_LINK_TRAINING_SELECT_DEFAULT   0x0
+#define NV_SWITCH_REGKEY_LINK_TRAINING_SELECT_NON_ALI   0x1
+#define NV_SWITCH_REGKEY_LINK_TRAINING_SELECT_ALI       0x2
 /*
  * NV_SWITCH_REGKEY_I2C_ACCESS_CONTROL - Enable access to all I2C Ports/Devices
  *
@@ -547,35 +544,4 @@
 #define NV_SWITCH_REGKEY_CRC_BIT_ERROR_RATE_LONG_TIMESCALE_MAN         6:4
 #define NV_SWITCH_REGKEY_CRC_BIT_ERROR_RATE_LONG_TIMESCALE_EXP         12:8
 
-/*
- * NV_SWITCH_REGKEY_BLOCK_CODE_MODE - Indicates the Forward Error Correction Mode
- *
- * Forward Error Correction Mode (Pre-HS).
- * DEFAULT = System Default
- * OFF = 0x0
- * ECC96_ENABLED = 0x1
- * ECC88_ENABLED = 0x2
- */
-#define NV_SWITCH_REGKEY_BLOCK_CODE_MODE                   "BlockCodeMode"
-#define NV_SWITCH_REGKEY_BLOCK_CODE_MODE_DEFAULT           0x0
-#define NV_SWITCH_REGKEY_BLOCK_CODE_MODE_OFF               0x0
-#define NV_SWITCH_REGKEY_BLOCK_CODE_MODE_ECC96_ENABLED     0x1
-#define NV_SWITCH_REGKEY_BLOCK_CODE_MODE_ECC88_ENABLED     0x2
-
-/*
- * NV_SWITCH_REGKEY_REFERENCE_CLOCK_MODE - Indicates the reference clock mode for 
- * the system w.r.t. this link.
- *
- * DEFAULT = System Default
- * COMMON = Common reference clock. Spread Spectrum (SS) may or may not be enabled.
- * NON_COMMON_NO_SS = Non-common reference clock without SS enabled.
- * NON_COMMON_SS = Non-common reference clock with SS enabled.
- */
-#define NV_SWITCH_REGKEY_REFERENCE_CLOCK_MODE                   "ReferenceClockMode"
-#define NV_SWITCH_REGKEY_REFERENCE_CLOCK_MODE_DEFAULT           0x0
-#define NV_SWITCH_REGKEY_REFERENCE_CLOCK_MODE_COMMON            0x0
-#define NV_SWITCH_REGKEY_REFERENCE_CLOCK_MODE_RESERVED          0x1
-#define NV_SWITCH_REGKEY_REFERENCE_CLOCK_MODE_NON_COMMON_NO_SS  0x2
-#define NV_SWITCH_REGKEY_REFERENCE_CLOCK_MODE_NON_COMMON_SS     0x3
-
 #endif //_REGKEY_NVSWITCH_H_
