@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, NVIDIA CORPORATION. All rights reserved.
+ * Copyright (c) 2021-2022, NVIDIA CORPORATION. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,13 +31,6 @@ extern "C" {
 
 #define  NVC770_DISPLAY                                             (0x0000C770)
 
-typedef struct
-{
-    NvU32   numHeads; // Number of HEADs in this chip/display
-    NvU32   numSors;  // Number of SORs in this chip/display
-    NvU32   numDsis;  // Number of DSIs in this chip/display
-} NVC770_ALLOCATION_PARAMETERS;
-
 #ifdef __cplusplus
 };     /* extern "C" */
 #endif
