@@ -1129,7 +1129,8 @@ static NvBool FrameLockDpyCanBeServer(const NVDpyEvoRec *pDpyEvo)
     NV30F1_CTRL_GSYNC_GET_CONTROL_SYNC_PARAMS gsyncGetControlSyncParams = { 0 };
     NVDispEvoPtr pDispEvo = pDpyEvo->pDispEvo;
     NVFrameLockEvoPtr pFrameLockEvo = pDispEvo->pFrameLockEvo;
-    const NvU32 head = pDpyEvo->head;
+    /* XXX[2Heads1OR] Get the primary hardware head. */
+    const NvU32 head = pDpyEvo->apiHead;
     const NVDispHeadStateEvoRec *pHeadState;
     NvU32 ret;
 
@@ -2171,7 +2172,8 @@ NvU32 nvGetFramelockServerHead(const NVDispEvoRec *pDispEvo)
 {
     const NVDpyEvoRec *pDpyEvo =
         nvGetDpyEvoFromDispEvo(pDispEvo, pDispEvo->framelock.server);
-    return (pDpyEvo != NULL) ? pDpyEvo->head : NV_INVALID_HEAD;
+    /* XXX[2Heads1OR] Get the primary hardware head. */
+    return (pDpyEvo != NULL) ? pDpyEvo->apiHead : NV_INVALID_HEAD;
 }
 
 NvU32 nvGetFramelockClientHeadsMask(const NVDispEvoRec *pDispEvo)
@@ -2179,11 +2181,15 @@ NvU32 nvGetFramelockClientHeadsMask(const NVDispEvoRec *pDispEvo)
     NvU32 headsMask = 0x0;
     const NVDpyEvoRec *pDpyEvo;
 
+    /*
+     * XXX[2Heads1OR] Translate api-head -> hardware-heads, and make sure to
+     * include the secondary hardware-head of the server dpy.
+     */
     FOR_ALL_EVO_DPYS(pDpyEvo, pDispEvo->framelock.clients, pDispEvo) {
-        if (pDpyEvo->head == NV_INVALID_HEAD) {
+        if (pDpyEvo->apiHead == NV_INVALID_HEAD) {
             continue;
         }
-        headsMask |= NVBIT(pDpyEvo->head);
+        headsMask |= NVBIT(pDpyEvo->apiHead);
     }
     return headsMask;
 }
@@ -2199,7 +2205,8 @@ void nvUpdateGLSFramelock(const NVDispEvoRec *pDispEvo, const NvU32 head,
      * apiHead -> pDpyEvo mapping will get implemented.
      */
     FOR_ALL_EVO_DPYS(pDpyEvo, pDispEvo->validDisplays, pDispEvo) {
-        if (pDpyEvo->head != head) {
+        /* XXX[2Heads1OR] Get the primary hardware head. */
+        if (pDpyEvo->apiHead != head) {
             continue;
         }
 
