@@ -466,10 +466,6 @@
 #define ESC_4000_G4_DEVID                  0xA1C1
 #define ESC_4000_G4_SSDEVID                0x871E
 
-// Lenovo Tomcat Workstation
-#define LENOVO_TOMCAT_DEVID                0x1B81
-#define LENOVO_TOMCAT_SSDEVID              0x104e
-
 
 // NVIDIA C51
 #define NVIDIA_C51_DEVICE_ID_MIN                        0x2F0
@@ -629,6 +625,7 @@ enum {
 ,     CS_NVIDIA_T186
 ,     CS_NVIDIA_T194
 ,     CS_NVIDIA_T234
+,     CS_NVIDIA_T23x
 ,     CS_MARVELL_THUNDERX2
 ,     CS_REDHAT_QEMU
 ,     CS_AMPERE_EMAG
