@@ -58,6 +58,7 @@
 #include <nvSha256.h>
 #include <gpu/gsp/kernel_gsp.h>
 #include <logdecode.h>
+#include <gpu/fsp/kern_fsp.h>
 
 #include <mem_mgr/virt_mem_mgr.h>
 
@@ -1655,6 +1656,17 @@ NvBool RmInitAdapter(
         goto shutdown;
     }
 
+    KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
+    if ((pKernelFsp != NULL) && !IS_GSP_CLIENT(pGpu) && !IS_VIRTUAL(pGpu))
+    {
+        status.rmStatus = kfspSendBootCommands_HAL(pGpu, pKernelFsp);
+        if (status.rmStatus != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "FSP boot command failed.\n");
+            goto shutdown;
+        }
+    }
+
     RmSetConsolePreservationParams(pGpu);
 
     //
@@ -1830,7 +1842,7 @@ NvBool RmInitAdapter(
     RmInitS0ixPowerManagement(nv);
     RmInitDeferredDynamicPowerManagement(nv);
 
-    if (!NV_IS_SOC_DISPLAY_DEVICE(nv))
+    if (!NV_IS_SOC_DISPLAY_DEVICE(nv) && !NV_IS_SOC_IGPU_DEVICE(nv))
     {
         status.rmStatus = RmRegisterGpudb(pGpu);
         if (status.rmStatus != NV_OK)
