@@ -50,10 +50,13 @@
 #include "vgpu/vgpu_events.h"
 #include "virtualization/hypervisor/hypervisor.h"
 #include "finn_rm_api.h"
+#include "os/os.h"
 
 #define SDK_ALL_CLASSES_INCLUDE_FULL_HEADER
 #include "g_allclasses.h"
 #undef SDK_ALL_CLASSES_INCLUDE_FULL_HEADER
+#include "nverror.h"
+
 
 #define RPC_STRUCTURES
 #define RPC_GENERIC_UNION
@@ -267,9 +270,9 @@ static NV_STATUS _issueRpcLarge
 
     //
     // Copy the initial buffer
-    // Temporary black magic WAR for bug 3594082: reducing the size by 2
+    // Temporary black magic WAR for bug 3594082: reducing the size by 1
     //
-    entryLength = NV_MIN(bufSize, pRpc->maxRpcSize - 2);
+    entryLength = NV_MIN(bufSize, pRpc->maxRpcSize - 1);
 
     if ((NvU8 *)vgpu_rpc_message_header_v != pBuf8)
         portMemCopy(vgpu_rpc_message_header_v, entryLength, pBuf8, entryLength);
@@ -296,9 +299,9 @@ static NV_STATUS _issueRpcLarge
 
     //
     // Copy the remaining buffers
-    // Temporary black magic WAR for bug 3594082: reducing the size by 2
+    // Temporary black magic WAR for bug 3594082: reducing the size by 1
     //
-    entryLength = pRpc->maxRpcSize - sizeof(rpc_message_header_v) - 2;
+    entryLength = pRpc->maxRpcSize - sizeof(rpc_message_header_v) - 1;
     while (remainingSize != 0)
     {
         if (entryLength > remainingSize)
@@ -357,8 +360,7 @@ static NV_STATUS _issueRpcLarge
 
     pBuf8 = (NvU8 *)pBuffer;
     remainingSize = bufSize;
-    entryLength = NV_MIN(bufSize, vgpu_rpc_message_header_v->length);
-    NV_CHECK_OR_RETURN(LEVEL_ERROR, entryLength <= pRpc->maxRpcSize, NV_ERR_INVALID_STATE);
+    entryLength = NV_MIN(bufSize, pRpc->maxRpcSize);
 
     if (((NvU8 *)vgpu_rpc_message_header_v != pBuf8) && bBidirectional)
       portMemCopy(pBuf8, entryLength, vgpu_rpc_message_header_v, entryLength);
@@ -389,11 +391,8 @@ static NV_STATUS _issueRpcLarge
                 NV_ASSERT(0);
                 return nvStatus;
             }
-
-            entryLength = vgpu_rpc_message_header_v->length;
+            entryLength = vgpu_rpc_message_header_v->length - sizeof(rpc_message_header_v);
             NV_CHECK_OR_RETURN(LEVEL_ERROR, entryLength <= pRpc->maxRpcSize, NV_ERR_INVALID_STATE);
-            NV_CHECK_OR_RETURN(LEVEL_ERROR, entryLength >= sizeof(rpc_message_header_v), NV_ERR_INVALID_STATE);
-            entryLength -= sizeof(rpc_message_header_v);
 
             if (entryLength > remainingSize)
                 entryLength = remainingSize;
@@ -1000,6 +999,18 @@ NV_STATUS rpcGetStaticInfo_v1A_05(OBJGPU *pGpu, OBJRPC *pRpc)
     return status;
 }
 
+NV_STATUS rpcGetStaticInfo_v20_01(OBJGPU *pGpu, OBJRPC *pRpc)
+{
+    NV_STATUS status = NV_OK;
+    return status;
+}
+
+NV_STATUS rpcGetStaticInfo_v20_04(OBJGPU *pGpu, OBJRPC *pRpc)
+{
+    NV_STATUS status = NV_OK;
+    return status;
+}
+
 NV_STATUS rpcGetGspStaticInfo_v14_00(OBJGPU *pGpu, OBJRPC *pRpc)
 {
     NV_STATUS status = NV_ERR_NOT_SUPPORTED;
@@ -1262,7 +1273,10 @@ NV_STATUS rpcGspSetSystemInfo_v17_00
         {
             rpcInfo->simAccessBufPhysAddr = 0;
         }
+        rpcInfo->pcieAtomicsOpMask = GPU_GET_KERNEL_BIF(pGpu) ?
+            GPU_GET_KERNEL_BIF(pGpu)->osPcieAtomicsOpMask : 0U;
         rpcInfo->consoleMemSize = GPU_GET_MEMORY_MANAGER(pGpu)->Ram.ReservedConsoleDispMemSize;
+        rpcInfo->maxUserVa      = osGetMaxUserVa();
 
         OBJCL *pCl = SYS_GET_CL(SYS_GET_INSTANCE());
         if (pCl != NULL)
@@ -1639,9 +1653,9 @@ NV_STATUS rpcRmApiControl_GSP
 
     if (status != NV_OK)
     {
-        NV_PRINTF_COND((status == (NV_ERR_NOT_SUPPORTED || NV_ERR_OBJECT_NOT_FOUND)), LEVEL_INFO, LEVEL_WARNING,
-            "GspRmControl failed: hClient=0x%08x; hObject=0x%08x; cmd=0x%08x; paramsSize=0x%08x; paramsStatus=0x%08x; status=0x%08x\n",
-            hClient, hObject, cmd, paramsSize, rpc_params->status, status);
+        NV_PRINTF(LEVEL_WARNING,
+                  "GspRmControl failed: hClient=0x%08x; hObject=0x%08x; cmd=0x%08x; paramsSize=0x%08x; paramsStatus=0x%08x; status=0x%08x\n",
+                  hClient, hObject, cmd, paramsSize, rpc_params->status, status);
     }
 
 done:
