@@ -411,25 +411,9 @@ void kdispUnbindUnmapDispChannel_IMPL
     RsClient           *pClient     = RES_GET_CLIENT(pDispChannel);
     RmClient           *pRmClient   = dynamicCast(pClient, RmClient);
     RS_PRIV_LEVEL       privLevel   = rmclientGetCachedPrivilege(pRmClient);
-    RS_ITERATOR         ContextDmaIt;
 
-    // Unbind all context dmas bound to this channel
-    ContextDmaIt = clientRefIter(pClient, RES_GET_REF(GPU_RES_GET_DEVICE(pDispChannel)), classId(ContextDma), RS_ITERATE_DESCENDANTS, NV_TRUE);
-    while (clientRefIterNext(ContextDmaIt.pClient, &ContextDmaIt))
-    {
-        ContextDma *pContextDma;
-
-        pContextDma = dynamicCast(ContextDmaIt.pResourceRef->pResource, ContextDma);
-        if (pContextDma == NULL)
-            continue;
-
-        // Quickly skip unbound ContextDmas
-        if (ctxdmaIsBound(pContextDma))
-        {
-            // Ignore unbind status as the ContextDma may not be bound to this channel
-            (void)dispchnUnbindCtx(pDispChannel, pGpu, pContextDma);
-        }
-    }
+    // Unbind all ContextDmas from this channel
+    dispchnUnbindAllCtx(pGpu, pDispChannel);
 
     // Unmap the channel
     osUnmapGPU(pGpu->pOsGpuInfo, privLevel, pDispChannel->pControl,
@@ -590,15 +574,25 @@ dispchnGetByHandle_IMPL
 NV_STATUS
 dispchnBindCtx_IMPL
 (
-    DispChannel  *pDispChannel,
     OBJGPU       *pGpu,
-    ContextDma   *pContextDma
+    ContextDma   *pContextDma,
+    NvHandle     hChannel
 )
 {
+    RsClient     *pClient = RES_GET_CLIENT(pContextDma);
+    DispChannel  *pDispChannel = NULL;
     NV_STATUS rmStatus = NV_OK;
     KernelDisplay *pKernelDisplay;
     DisplayInstanceMemory *pInstMem;
 
+    // Look-up channel
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+        dispchnGetByHandle(pClient, hChannel, &pDispChannel));
+
+    // Ensure ContextDma and DisplayChannel are on the save device
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, pContextDma->pDevice == GPU_RES_GET_DEVICE(pDispChannel),
+                       NV_ERR_INVALID_DEVICE);
+
     //
     // Enforce alignment requirements
     // ISO  ctx dmas need to be a multiple of 256B and 256B aligned
@@ -633,16 +627,26 @@ dispchnBindCtx_IMPL
 NV_STATUS
 dispchnUnbindCtx_IMPL
 (
-    DispChannel *pDispChannel,
     OBJGPU      *pGpu,
-    ContextDma  *pContextDma
+    ContextDma  *pContextDma,
+    NvHandle     hChannel
 )
 {
+    RsClient     *pClient = RES_GET_CLIENT(pContextDma);
+    DispChannel  *pDispChannel = NULL;
     NV_STATUS  rmStatus = NV_OK;
     KernelDisplay *pKernelDisplay;
     DisplayInstanceMemory *pInstMem;
     NvBool bFound = NV_FALSE;
 
+    // Look-up channel given by client
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+        dispchnGetByHandle(pClient, hChannel, &pDispChannel));
+
+    // Ensure ContextDma and DisplayChannel are on the save device
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, pContextDma->pDevice == GPU_RES_GET_DEVICE(pDispChannel),
+                       NV_ERR_INVALID_DEVICE);
+
     SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY)
     pKernelDisplay = GPU_GET_KERNEL_DISPLAY(pGpu);
     pInstMem = KERNEL_DISPLAY_GET_INST_MEM(pKernelDisplay);
@@ -658,6 +662,50 @@ dispchnUnbindCtx_IMPL
     return bFound ? NV_OK : NV_ERR_INVALID_STATE;
 }
 
+/*!
+ * @brief Unbind all ContextDmas from the given channel
+ */
+void
+dispchnUnbindAllCtx_IMPL
+(
+    OBJGPU      *pGpu,
+    DispChannel *pDispChannel
+)
+{
+    KernelDisplay *pKernelDisplay;
+    DisplayInstanceMemory *pInstMem;
+
+    SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY)
+    pKernelDisplay = GPU_GET_KERNEL_DISPLAY(pGpu);
+    pInstMem = KERNEL_DISPLAY_GET_INST_MEM(pKernelDisplay);
+
+    instmemUnbindDispChannelContextDmas(pGpu, pInstMem, pDispChannel);
+
+    SLI_LOOP_END
+}
+
+/*!
+ * @brief Unbind ContextDma from all display channels
+ */
+void
+dispchnUnbindCtxFromAllChannels_IMPL
+(
+    OBJGPU      *pGpu,
+    ContextDma  *pContextDma
+)
+{
+    KernelDisplay *pKernelDisplay;
+    DisplayInstanceMemory *pInstMem;
+
+    SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY)
+    pKernelDisplay = GPU_GET_KERNEL_DISPLAY(pGpu);
+    pInstMem = KERNEL_DISPLAY_GET_INST_MEM(pKernelDisplay);
+
+    instmemUnbindContextDmaFromAllChannels(pGpu, pInstMem, pContextDma);
+
+    SLI_LOOP_END
+}
+
 NV_STATUS
 kdispSetPushBufferParamsToPhysical_IMPL
 (
