@@ -229,8 +229,7 @@ _nv8deCtrlCmdReadWriteSurface
             }
             else if (traceArg.aperture == ADDR_FBMEM)
             {
-                NV_ASSERT_OK_OR_RETURN(memdescCreate(&pMemDesc, pGpu, curSize, 0, NV_TRUE,
-                                                     traceArg.aperture, NV_MEMORY_UNCACHED, MEMDESC_FLAGS_NONE));
+                memdescCreate(&pMemDesc, pGpu, curSize, 0, NV_TRUE, traceArg.aperture, NV_MEMORY_UNCACHED, MEMDESC_FLAGS_NONE);
                 memdescDescribe(pMemDesc, traceArg.aperture, traceArg.pa, curSize);
             }
 
@@ -685,10 +684,6 @@ NV_STATUS ksmdbgssnCtrlCmdDebugExecRegOps_IMPL
     NV_STATUS status = NV_OK;
     NvBool isClientGspPlugin = NV_FALSE;
 
-    NV_CHECK_OR_RETURN(LEVEL_ERROR,
-        pParams->regOpCount <= NV83DE_CTRL_GPU_EXEC_REG_OPS_MAX_OPS,
-        NV_ERR_INVALID_ARGUMENT);
-
     // Check if User have permission to access register offset
     NV_CHECK_OK_OR_RETURN(LEVEL_INFO,
         gpuValidateRegOps(pGpu, pParams->regOps, pParams->regOpCount,
@@ -730,11 +725,9 @@ ksmdbgssnCtrlCmdDebugReadBatchMemory_IMPL
     {
         NV_STATUS localStatus = NV_OK;
         NvP64 pData = (NvP64)(((NvU8 *)pParams->pData) + pParams->entries[i].dataOffset);
-        NvU32 endingOffset;
 
         NV_CHECK_OR_ELSE(LEVEL_ERROR,
-            portSafeAddU32(pParams->entries[i].dataOffset, pParams->entries[i].length, &endingOffset) &&
-            (endingOffset <= pParams->dataLength),
+            pParams->entries[i].dataOffset < pParams->dataLength,
             localStatus = NV_ERR_INVALID_OFFSET;
             goto updateStatus; );
 
@@ -769,18 +762,13 @@ ksmdbgssnCtrlCmdDebugWriteBatchMemory_IMPL
     NV_STATUS status = NV_OK;
     NvU32 i;
 
-    NV_CHECK_OR_RETURN(LEVEL_ERROR, pParams->count <= MAX_ACCESS_MEMORY_OPS,
-                       NV_ERR_INVALID_ARGUMENT);
-
     for (i = 0; i < pParams->count; ++i)
     {
         NV_STATUS localStatus = NV_OK;
         NvP64 pData = (NvP64)(((NvU8 *)pParams->pData) + pParams->entries[i].dataOffset);
-        NvU32 endingOffset;
 
         NV_CHECK_OR_ELSE(LEVEL_ERROR,
-            portSafeAddU32(pParams->entries[i].dataOffset, pParams->entries[i].length, &endingOffset) &&
-            (endingOffset <= pParams->dataLength),
+            (pParams->entries[i].dataOffset + pParams->entries[i].length) <= pParams->dataLength,
             localStatus = NV_ERR_INVALID_OFFSET;
             goto updateStatus; );
 
