@@ -1332,6 +1332,14 @@ kgspInitRm_IMPL
      * Here we prepare the Booter ucode images in SYSMEM so they may be loaded onto
      * SEC2 (Load / Unload) and NVDEC0 (Unload).
      */
+    if (pKernelGsp->bPartitionedFmc)
+    {
+        //
+        // The secure boot ucode is included in the partitioned FMC, no need for
+        // separate Booter ucodes.
+        //
+    }
+    else
     {
         if (pKernelGsp->pBooterLoadUcode == NULL)
         {
@@ -1477,7 +1485,11 @@ kgspUnloadRm_IMPL
     //
     threadStateResetTimeout(pGpu);
 
-    kflcnReset_HAL(pGpu, staticCast(pKernelGsp, KernelFalcon));
+    // Because of COT, RM cannot reset GSP-RISCV and FSP has exclusive access to reset and reboot GSP for next run.
+    if(!(pGpu->getProperty(pGpu, PDB_PROP_GPU_IS_COT_ENABLED)))
+    {
+        kflcnReset_HAL(pGpu, staticCast(pKernelGsp, KernelFalcon));
+    }
 
     // Invoke FWSEC-SB to put back PreOsApps during driver unload
     status = kgspExecuteFwsecSb_HAL(pGpu, pKernelGsp, pKernelGsp->pFwsecUcode);
@@ -1487,6 +1499,15 @@ kgspUnloadRm_IMPL
         NV_ASSERT(0);
     }
 
+    if (pKernelGsp->bPartitionedFmc)
+    {
+        //
+        // GSP-RM invokes the partitioned FMC to unload directly as part of the
+        // NV_RM_RPC_UNLOADING_GUEST_DRIVER call above.
+        //
+        status = rpcStatus;
+    }
+    else
     {
         // After instructing GSP-RM to unload itself, run Booter Unload to teardown WPR2
         status = kgspExecuteBooterUnloadIfNeeded_HAL(pGpu, pKernelGsp);
