@@ -38,7 +38,6 @@
 #include "nvVer.h"
 #include "gpu/bif/kernel_bif.h"
 #include "gpu/bus/kern_bus.h"
-#include "gpu/disp/kern_disp.h"
 #include "gpu/mmu/kern_gmmu.h"
 #include "kernel/gpu/intr/intr.h"
 #include "kernel/gpu/mc/kernel_mc.h"
@@ -279,18 +278,6 @@ getGpuInfos(Subdevice *pSubdevice, NV2080_CTRL_GPU_GET_INFO_V2_PARAMS *pParams,
                 }
                 break;
             }
-            case NV2080_CTRL_GPU_INFO_INDEX_DISPLAY_ENABLED:
-            {
-                if (GPU_GET_KERNEL_DISPLAY(pGpu) != NULL)
-                {
-                    data = NV2080_CTRL_GPU_INFO_DISPLAY_ENABLED_YES;
-                }
-                else
-                {
-                    data = NV2080_CTRL_GPU_INFO_DISPLAY_ENABLED_NO;
-                }
-                break;
-            }
             case NV2080_CTRL_GPU_INFO_INDEX_MOBILE_CONFIG_ENABLED:
             {
                 if (IsMobile(pGpu))
@@ -1786,19 +1773,6 @@ subdeviceCtrlCmdGpuGetPidInfo_IMPL
     return NV_OK;
 }
 
-NV_STATUS
-subdeviceCtrlCmdGpuInterruptFunction_IMPL
-(
-    Subdevice *pSubdevice,
-    NV2080_CTRL_GPU_INTERRUPT_FUNCTION_PARAMS *pParams
-)
-{
-    OBJGPU *pGpu = GPU_RES_GET_GPU(pSubdevice);
-    Intr *pIntr = GPU_GET_INTR(pGpu);
-
-    return intrTriggerCpuDoorbellForVF_HAL(pGpu, pIntr, pParams->gfid);
-}
-
 // Control call to fetch the Runlist pri base for the engine(s) specified
 NV_STATUS
 subdeviceCtrlCmdGpuGetEngineRunlistPriBase_IMPL
