@@ -40,41 +40,6 @@
 
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
 
-typedef struct
-{
-    EVENTNOTIFICATION *pEventNotify;
-    Memory *pMemory;
-    ListNode eventNotificationListNode;
-    ListNode pendingEventNotifyListNode;
-} ENGINE_EVENT_NOTIFICATION;
-
-//
-// These lists are intrusive to avoid memory allocation during insertion while
-// in a non-preemptible context (holding a spinlock/in an ISR).
-//
-MAKE_INTRUSIVE_LIST(EngineEventNotificationList, ENGINE_EVENT_NOTIFICATION,
-                    eventNotificationListNode);
-MAKE_INTRUSIVE_LIST(PendingEventNotifyList, ENGINE_EVENT_NOTIFICATION,
-                    pendingEventNotifyListNode);
-
-// Linked list of per engine non-stall event notifications
-struct GpuEngineEventNotificationList
-{
-    PORT_SPINLOCK *pSpinlock;
-
-    // List insertion and removal happens under pSpinlock
-    EngineEventNotificationList eventNotificationList;
-
-    // Filled while pSpinlock is held, drained outside of the lock in ISR
-    PendingEventNotifyList pendingEventNotifyList;
-
-    //
-    // Set to non-zero under pSpinlock, set to zero outside of the lock in ISR
-    // Insertions/removals on eventNotificationList are blocked while non-zero
-    //
-    volatile NvU32 pendingEventNotifyCount;
-};
-
 static NV_STATUS _insertEventNotification
 (
     PEVENTNOTIFICATION *ppEventNotification,
@@ -102,283 +67,159 @@ static NV_STATUS _removeEventNotification
 //
 //---------------------------------------------------------------------------
 
-NV_STATUS gpuEngineEventNotificationListCreate
+static NV_STATUS engineNonStallEventOp
 (
     OBJGPU *pGpu,
-    GpuEngineEventNotificationList **ppEventNotificationList
+    NvU32 engineId,
+    PEVENTNOTIFICATION pEventNotify,
+    Memory *pMemory,
+    NvBool bInsert
 )
 {
-    NV_STATUS status = NV_OK;
+    ENGINE_EVENT_NODE *pTempNode;
+    NvBool bFound = NV_FALSE;
 
-    PORT_MEM_ALLOCATOR *pAllocator = portMemAllocatorGetGlobalNonPaged();
-    GpuEngineEventNotificationList *pEventNotificationList =
-        portMemAllocNonPaged(sizeof(*pEventNotificationList));
-    NV_ASSERT_OR_RETURN(pEventNotificationList != NULL, NV_ERR_NO_MEMORY);
-
-    portMemSet(pEventNotificationList, 0, sizeof(*pEventNotificationList));
-
-    pEventNotificationList->pSpinlock = portSyncSpinlockCreate(pAllocator);
-    NV_ASSERT_OR_ELSE(pEventNotificationList->pSpinlock != NULL,
+    if (bInsert)
     {
-        status = NV_ERR_INSUFFICIENT_RESOURCES;
-        goto exit;
-    });
+        pTempNode = portMemAllocNonPaged(sizeof(ENGINE_EVENT_NODE));
 
-    listInitIntrusive(&pEventNotificationList->eventNotificationList);
-    listInitIntrusive(&pEventNotificationList->pendingEventNotifyList);
+        if (pTempNode == NULL)
+            return NV_ERR_NO_MEMORY;
 
-    portAtomicSetU32(&pEventNotificationList->pendingEventNotifyCount, 0);
+        // Acquire engine list spinlock before adding to engine event list
+        portSyncSpinlockAcquire(pGpu->engineNonstallIntr[engineId].pSpinlock);
+        pTempNode->pNext = pGpu->engineNonstallIntr[engineId].pEventNode;
+        pTempNode->pEventNotify = pEventNotify;
+        pTempNode->pMemory = pMemory;
 
-    *ppEventNotificationList = pEventNotificationList;
+        pGpu->engineNonstallIntr[engineId].pEventNode = pTempNode;
 
-exit:
-    if (status != NV_OK)
-        gpuEngineEventNotificationListDestroy(pGpu, pEventNotificationList);
-    return status;
-}
-
-void gpuEngineEventNotificationListDestroy
-(
-    OBJGPU *pGpu,
-    GpuEngineEventNotificationList *pEventNotificationList
-)
-{
-    if (pEventNotificationList == NULL)
-        return;
-
-    NV_ASSERT(pEventNotificationList->pendingEventNotifyCount == 0);
-
-    NV_ASSERT(listCount(&pEventNotificationList->pendingEventNotifyList) == 0);
-    listDestroy(&pEventNotificationList->pendingEventNotifyList);
-
-    NV_ASSERT(listCount(&pEventNotificationList->eventNotificationList) == 0);
-    listDestroy(&pEventNotificationList->eventNotificationList);
-
-    if (pEventNotificationList->pSpinlock != NULL)
-        portSyncSpinlockDestroy(pEventNotificationList->pSpinlock);
-
-    portMemFree(pEventNotificationList);
-}
-
-static void _gpuEngineEventNotificationListLockPreemptible
-(
-    GpuEngineEventNotificationList *pEventNotificationList
-)
-{
-    do
-    {
-        portSyncSpinlockAcquire(pEventNotificationList->pSpinlock);
-
-        //
-        // Only return with the lock held once there are no pending
-        // notifications to process. No more pending notifications can be queued
-        // while the spinlock is held, and we drop the lock to re-enable
-        // preemption, to guarantee that _gpuEngineEventNotificationListNotify()
-        // can make forward progress to drain the pending notifications list.
-        //
-        if (pEventNotificationList->pendingEventNotifyCount == 0)
-            return;
-
-        portSyncSpinlockRelease(pEventNotificationList->pSpinlock);
-
-        //
-        // Spin waiting for the pending notifications to drain.
-        // This can only be done in a preemptible context (i.e., add
-        // or remove notification in a thread context).
-        //
-        while (pEventNotificationList->pendingEventNotifyCount > 0)
-            osSpinLoop();
-    } while (NV_TRUE);
-}
-
-static inline void _gpuEngineEventNotificationListUnlockPreemptible
-(
-    GpuEngineEventNotificationList *pEventNotificationList
-)
-{
-    portSyncSpinlockRelease(pEventNotificationList->pSpinlock);
-}
-
-static NV_STATUS _gpuEngineEventNotificationInsert
-(
-    GpuEngineEventNotificationList *pEventNotificationList,
-    EVENTNOTIFICATION *pEventNotify,
-    Memory *pMemory
-)
-{
-    NV_CHECK_OR_RETURN(LEVEL_ERROR, pEventNotify != NULL,
-                       NV_ERR_INVALID_ARGUMENT);
-
-    // Allocate the new node outside of the spinlock
-    ENGINE_EVENT_NOTIFICATION *pEngineEventNotification =
-        portMemAllocNonPaged(sizeof(*pEngineEventNotification));
-
-    NV_CHECK_OR_RETURN(LEVEL_ERROR, pEngineEventNotification != NULL,
-                       NV_ERR_NO_MEMORY);
-
-    portMemSet(pEngineEventNotification, 0, sizeof(*pEngineEventNotification));
-
-    pEngineEventNotification->pEventNotify = pEventNotify;
-    pEngineEventNotification->pMemory = pMemory;
-
-    // Take the lock to add the node to the list
-    _gpuEngineEventNotificationListLockPreemptible(pEventNotificationList);
+        // Release engine list spinlock
+        portSyncSpinlockRelease(pGpu->engineNonstallIntr[engineId].pSpinlock);
+    }
+    else
     {
-        listPrependExisting(&pEventNotificationList->eventNotificationList,
-                            pEngineEventNotification);
+        ENGINE_EVENT_NODE *pEngNode, *pPrevNode = NULL;
+
+        // Acquire engine list spinlock before traversing engine event list
+        portSyncSpinlockAcquire(pGpu->engineNonstallIntr[engineId].pSpinlock);
+
+        pEngNode = pGpu->engineNonstallIntr[engineId].pEventNode;
+        while (pEngNode)
+        {
+            if (pEngNode->pEventNotify == pEventNotify)
+            {
+                if (pPrevNode == NULL)
+                    pGpu->engineNonstallIntr[engineId].pEventNode = pEngNode->pNext;
+                else
+                    pPrevNode->pNext = pEngNode->pNext;
+
+                pTempNode = pEngNode;
+                bFound = NV_TRUE;
+                break;
+            }
+            else
+            {
+                pPrevNode = pEngNode;
+            }
+            pEngNode = pEngNode->pNext;
+        }
+
+        // Release engine list spinlock
+        portSyncSpinlockRelease(pGpu->engineNonstallIntr[engineId].pSpinlock);
+
+        if (bFound)
+        {
+            portMemFree(pTempNode);
+        }
+        else
+        {
+            NV_ASSERT_FAILED("failed to find non-stall event!");
+            return NV_ERR_INVALID_STATE;
+        }
     }
-    _gpuEngineEventNotificationListUnlockPreemptible(pEventNotificationList);
 
     return NV_OK;
 }
 
-static void _gpuEngineEventNotificationRemove
-(
-    GpuEngineEventNotificationList *pEventNotificationList,
-    EVENTNOTIFICATION *pEventNotify
-)
+static NV_STATUS _engineNonStallIntrNotifyImpl(OBJGPU *pGpu, NvU32 engineId, NvHandle hEvent)
 {
-    ENGINE_EVENT_NOTIFICATION *pEngineEventNotification = NULL;
-
-    _gpuEngineEventNotificationListLockPreemptible(pEventNotificationList);
-    {
-        EngineEventNotificationListIter it =
-            listIterAll(&pEventNotificationList->eventNotificationList);
-        while (listIterNext(&it))
-        {
-            if (it.pValue->pEventNotify == pEventNotify)
-            {
-                pEngineEventNotification = it.pValue;
-                listRemove(&pEventNotificationList->eventNotificationList,
-                           pEngineEventNotification);
-                break;
-            }
-        }
-    }
-    _gpuEngineEventNotificationListUnlockPreemptible(pEventNotificationList);
-
-    NV_ASSERT(pEngineEventNotification != NULL);
-    portMemFree(pEngineEventNotification);
-}
-
-static NV_STATUS _gpuEngineEventNotificationListNotify
-(
-    OBJGPU *pGpu,
-    GpuEngineEventNotificationList *pEventNotificationList,
-    NvHandle hEvent
-)
-{
-    NV_STATUS status = NV_OK;
-    PendingEventNotifyList *pPending =
-        &pEventNotificationList->pendingEventNotifyList;
+    ENGINE_EVENT_NODE *pTempHead;
+    Memory *pSemMemory;
+    NvU32 semValue;
+    NvU32 *pTempKernelMapping = NULL;
+    NV_STATUS rmStatus = NV_OK;
 
     //
     // Acquire engine list spinlock before traversing the list. Note that this
     // is called without holding locks from ISR for Linux. This spinlock is used
-    // to protect the per GPU per engine event node list.
+    // to protect per GPU per engine event node list.
     //
-    portSyncSpinlockAcquire(pEventNotificationList->pSpinlock);
+    portSyncSpinlockAcquire(pGpu->engineNonstallIntr[engineId].pSpinlock);
+
+    pTempHead = pGpu->engineNonstallIntr[engineId].pEventNode;
+    while (pTempHead)
     {
-        // We don't expect this to be called multiple times in parallel
-        NV_ASSERT_OR_ELSE(pEventNotificationList->pendingEventNotifyCount == 0,
+        if (!pTempHead->pEventNotify)
         {
-            portSyncSpinlockRelease(pEventNotificationList->pSpinlock);
-            return NV_ERR_INVALID_STATE;
-        });
-
-        EngineEventNotificationListIter it =
-            listIterAll(&pEventNotificationList->eventNotificationList);
-        while (listIterNext(&it))
-        {
-            ENGINE_EVENT_NOTIFICATION *pEngineEventNotification = it.pValue;
-            if (hEvent &&
-                pEngineEventNotification->pEventNotify->hEvent != hEvent)
-                continue;
-
-            Memory *pSemMemory = pEngineEventNotification->pMemory;
-            if (pSemMemory &&
-                pSemMemory->vgpuNsIntr.isSemaMemValidationEnabled &&
-                pSemMemory->pMemDesc && pSemMemory->pMemDesc->Allocated)
-            {
-                NvU32 *pTempKernelMapping =
-                    (NvU32 *)NvP64_VALUE(
-                        memdescGetKernelMapping(pSemMemory->pMemDesc));
-                if (pTempKernelMapping == NULL)
-                {
-                    NV_PRINTF(LEVEL_WARNING,
-                        "Per-vGPU semaphore location mapping is NULL."
-                        " Skipping the current node.\n");
-                    continue;
-                }
-
-                NvU32 semValue = MEM_RD32(pTempKernelMapping +
-                                          (pSemMemory->vgpuNsIntr.nsSemOffset /
-                                           sizeof(NvU32)));
-
-                if (pSemMemory->vgpuNsIntr.nsSemValue == semValue)
-                    continue;
-
-                pSemMemory->vgpuNsIntr.nsSemValue = semValue;
-
-            }
-
-            //
-            // Queue up this event notification to be completed outside of the
-            // critical section, as the osNotifyEvent implementation may need
-            // to be preemptible.
-            //
-            listAppendExisting(pPending, pEngineEventNotification);
+            rmStatus = NV_ERR_INVALID_STATE;
+            break;
         }
 
-        portAtomicSetU32(&pEventNotificationList->pendingEventNotifyCount,
-                         listCount(pPending));
-    }
-    portSyncSpinlockRelease(pEventNotificationList->pSpinlock);
+        if (hEvent && pTempHead->pEventNotify->hEvent != hEvent)
+            goto nextEvent;
 
-    //
-    // Iterate through the pending notifications and call the OS to send them.
-    // Note that osNotifyEvent may need to be preemptible, so this is done
-    // outside of the spinlock-protected critical section. 
-    //
-    PendingEventNotifyListIter it = listIterAll(pPending);
-    while (listIterNext(&it))
-    {
-        NV_ASSERT_OK_OR_CAPTURE_FIRST_ERROR(status,
-            osNotifyEvent(pGpu, it.pValue->pEventNotify, 0, 0, NV_OK));
+        pSemMemory = pTempHead->pMemory;
+
+        if (pSemMemory && pSemMemory->vgpuNsIntr.isSemaMemValidationEnabled &&
+            pSemMemory->pMemDesc && pSemMemory->pMemDesc->Allocated)
+        {
+            pTempKernelMapping = (NvU32 *)NvP64_VALUE(memdescGetKernelMapping(pSemMemory->pMemDesc));
+            if (pTempKernelMapping == NULL)
+            {
+                NV_PRINTF(LEVEL_WARNING, "Per-vGPU semaphore location mapping is NULL. Skipping the current node.\n");
+                pTempHead = pTempHead->pNext;
+                continue;
+            }
+            semValue = MEM_RD32(pTempKernelMapping + (pSemMemory->vgpuNsIntr.nsSemOffset / sizeof(NvU32)));
+
+            if (pSemMemory->vgpuNsIntr.nsSemValue == semValue)
+            {
+                pTempHead = pTempHead->pNext;
+                continue;
+            }
+
+            pSemMemory->vgpuNsIntr.nsSemValue = semValue;
+
+        }
+
+        if (osNotifyEvent(pGpu, pTempHead->pEventNotify, 0, 0, NV_OK) != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "failed to notify event for engine 0x%x\n",
+                      engineId);
+            NV_ASSERT(0);
+            rmStatus = NV_ERR_INVALID_STATE;
+            break;
+        }
+
+    nextEvent:
+        pTempHead = pTempHead->pNext;
     }
 
-    // Remove all entries from the pending event notify list
-    ENGINE_EVENT_NOTIFICATION *pIter, *pNext;
-    for (pIter = listHead(pPending); pIter != NULL; pIter = pNext)
-    {
-        pNext = listNext(pPending, pIter);
-        listRemove(pPending, pIter);
-    }
-
-    NV_ASSERT(listCount(pPending) == 0);
-
-    // Mark the event notifications as drained so preemptible code can continue
-    portAtomicSetU32(&pEventNotificationList->pendingEventNotifyCount, 0);
-
-    return status;
+    portSyncSpinlockRelease(pGpu->engineNonstallIntr[engineId].pSpinlock);
+    return rmStatus;
 }
 
 NV_STATUS
 engineNonStallIntrNotify(OBJGPU *pGpu, NvU32 engineId)
 {
-    NV_ASSERT_OR_RETURN(engineId < NV_ARRAY_ELEMENTS(pGpu->engineNonstallIntrEventNotifications),
-                        NV_ERR_INVALID_ARGUMENT);
-    return _gpuEngineEventNotificationListNotify(pGpu,
-        pGpu->engineNonstallIntrEventNotifications[engineId], 0);
+    return _engineNonStallIntrNotifyImpl(pGpu, engineId, 0);
 }
 
 NV_STATUS
 engineNonStallIntrNotifyEvent(OBJGPU *pGpu, NvU32 engineId, NvHandle hEvent)
 {
-    NV_ASSERT_OR_RETURN(engineId < NV_ARRAY_ELEMENTS(pGpu->engineNonstallIntrEventNotifications),
-                        NV_ERR_INVALID_ARGUMENT);
-    return _gpuEngineEventNotificationListNotify(pGpu,
-        pGpu->engineNonstallIntrEventNotifications[engineId], hEvent);
+    return _engineNonStallIntrNotifyImpl(pGpu, engineId, hEvent);
 }
 
 static NV_STATUS
@@ -469,6 +310,15 @@ eventGetEngineTypeFromSubNotifyIndex
         case NV2080_NOTIFIERS_NVDEC4:
             *engineIdx = NV2080_ENGINE_TYPE_NVDEC4;
             break;
+        case NV2080_NOTIFIERS_NVDEC5:
+            *engineIdx = NV2080_ENGINE_TYPE_NVDEC5;
+            break;
+        case NV2080_NOTIFIERS_NVDEC6:
+            *engineIdx = NV2080_ENGINE_TYPE_NVDEC6;
+            break;
+        case NV2080_NOTIFIERS_NVDEC7:
+            *engineIdx = NV2080_ENGINE_TYPE_NVDEC7;
+            break;
         case NV2080_NOTIFIERS_PDEC:
             *engineIdx = NV2080_ENGINE_TYPE_VP;
             break;
@@ -489,6 +339,27 @@ eventGetEngineTypeFromSubNotifyIndex
         case NV2080_NOTIFIERS_NVJPEG0:
             *engineIdx = NV2080_ENGINE_TYPE_NVJPEG0;
             break;
+        case NV2080_NOTIFIERS_NVJPEG1:
+            *engineIdx = NV2080_ENGINE_TYPE_NVJPEG1;
+            break;
+        case NV2080_NOTIFIERS_NVJPEG2:
+            *engineIdx = NV2080_ENGINE_TYPE_NVJPEG2;
+            break;
+        case NV2080_NOTIFIERS_NVJPEG3:
+            *engineIdx = NV2080_ENGINE_TYPE_NVJPEG3;
+            break;
+        case NV2080_NOTIFIERS_NVJPEG4:
+            *engineIdx = NV2080_ENGINE_TYPE_NVJPEG4;
+            break;
+        case NV2080_NOTIFIERS_NVJPEG5:
+            *engineIdx = NV2080_ENGINE_TYPE_NVJPEG5;
+            break;
+        case NV2080_NOTIFIERS_NVJPEG6:
+            *engineIdx = NV2080_ENGINE_TYPE_NVJPEG6;
+            break;
+        case NV2080_NOTIFIERS_NVJPEG7:
+            *engineIdx = NV2080_ENGINE_TYPE_NVJPEG7;
+            break;
         case NV2080_NOTIFIERS_OFA:
             *engineIdx = NV2080_ENGINE_TYPE_OFA;
             break;
@@ -587,9 +458,8 @@ NV_STATUS registerEventNotification
                 free_entry);
         }
 
-        rmStatus = _gpuEngineEventNotificationInsert(
-                        pGpu->engineNonstallIntrEventNotifications[engineId],
-                        *ppEventNotification, pSemMemory);
+        rmStatus = engineNonStallEventOp(pGpu, engineId,
+                        *ppEventNotification, pSemMemory, NV_TRUE);
 
         if (rmStatus != NV_OK)
             goto free_entry;
@@ -790,9 +660,8 @@ NV_STATUS unregisterEventNotificationWithData
             engineId = globalEngineId;
         }
 
-        _gpuEngineEventNotificationRemove(
-            pGpu->engineNonstallIntrEventNotifications[engineId],
-            pTargetEvent);
+        rmStatus = engineNonStallEventOp(pGpu, engineId,
+                        pTargetEvent, NULL, NV_FALSE);
     }
 
 free_entry:
